#include <Adafruit_NeoPixel.h>
#include <Adafruit_GFX.h>
#include <Adafruit_NeoMatrix.h>
#ifdef __AVR__
#include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif
// INCLUDE LIBRARY
#include <AlignedJoy.h>
#include <ezBuzzer.h>


// PINs DEFINITION
// joystick 1
#define PIN_JOY1_X   A0  //(up  down)
#define PIN_JOY1_Y   A1  //(left  right)
// joystick 2
#define PIN_JOY2_X   A3  //(up  down)
#define PIN_JOY2_Y   A2  //(left  right)
#define PUSH1 2
#define PUSH2 4
#define BUZZER_PIN 8
#define FLUSH true
#define NOFLUSH false

#define PRESSED 1
#define RELEASED 0

#define UP 0
#define DOWN 1
#define LEFT 2
#define RIGHT 3
#define CENTER 4
#define INITIALDELAY 500
#define SPEEDUP 0.9
#define MINDELAY 180

#define LED_PIN    7
#define LED_COUNT 64
#define BLACK    0x0000
#define BLUE     0x001F
#define DARKBLUE 0x0012
#define RED      0xA800
#define GREEN    0x07E0
#define CYAN     0x07FF
#define MAGENTA  0xF81F
#define YELLOW   0xFFE0
#define WHITE    0xFFFF
#define GREY     0x2222


#define APPLE_COLOR RED
#define TICTACTOEGRID DARKBLUE

#define SNAKE2PLAYERS 0
#define TICTACTOE 1
#define SNAKE1PLAYER 2
#define TOGGLESOUND 3
#define MINMODE 0
#define MAXMODE 3


// CLASS CONSTRUCTOR
// new joystick object
AlignedJoy joystick_1(PIN_JOY1_X, PIN_JOY1_Y);
AlignedJoy joystick_2(PIN_JOY2_X, PIN_JOY2_Y);

// Declare our NeoPixel strip object:
Adafruit_NeoMatrix matrix = Adafruit_NeoMatrix(8, 8, LED_PIN,
                            NEO_MATRIX_BOTTOM     + NEO_MATRIX_LEFT +
                            NEO_MATRIX_ROWS + NEO_MATRIX_ZIGZAG,
                            NEO_GRB            + NEO_KHZ800);

ezBuzzer buzzer(BUZZER_PIN);

bool processed = false;
bool soundon = true;
short dir[2];
unsigned short int applex, appley, oldapplex, oldappley;
unsigned long int last_moved;
unsigned long int last_toggled;
unsigned int mydelay;
bool joysticks_moved = false, joy1_moved = false, joy2_moved = false;
bool joy1_pushed = false, joy2_pushed = false;
short joy1_direction = CENTER, joy2_direction = CENTER;
short oldStatePB1, oldStatePB2;
short mode;


// SOUNDS
int start[] {  NOTE_C6, NOTE_C7 };
int start_durations[] {  12, 3 };
int snake_start[] = {  NOTE_C4, NOTE_D4, NOTE_E4,  NOTE_D4, NOTE_C4, NOTE_D4,  NOTE_E4, NOTE_C4, NOTE_C4 };
int snake_start_durations[] = {  14, 14, 14, 14, 14, 14, 7, 7, 7 };
int snake_eaten_apple[] = {  NOTE_C3, NOTE_C3, NOTE_C3, NOTE_E3 };
int snake_eaten_apple_durations[] = {  14, 14, 14, 14 };
int snake_dead[] = {  NOTE_E2, NOTE_F2, NOTE_G2, NOTE_E2, NOTE_F2, NOTE_D2, NOTE_C2 };
int snake_dead_durations[] = {  14, 14, 14, 14, 14, 14, 14 };
int tictactoe_p1[] = {NOTE_DS5};
int tictactoe_p2[] = {NOTE_C4};
int tictactoe_pdur[] = {8};
int notify1[] = {  NOTE_G2, NOTE_G3 };
int notify1_durations[] = {  12, 12 };


typedef struct {
  unsigned short int x;
  unsigned short int y;
} coordinates;

// SNAKE STUFF
typedef struct {
  uint16_t colors[7] = {};
  coordinates pos[7] = {};
  unsigned short len;
  unsigned short dir;
} snake;

snake snakes[2];
bool snake_is_dead[2];
short snake_count;

// TICTACTOE STUFF
short selected;
short count;
bool showing;
uint16_t activecolor;
short activeplayer;
coordinates grid_starts[] = {{0, 0}, {3, 0}, {6, 0}, {0, 3}, {3, 3}, {6, 3}, {0, 6}, {3, 6}, {6, 6}};
uint16_t grid[9];
short wingrid[9];
short winningplayer;
bool win;

void setapplexy(int myx, int myy) {
  matrix.drawPixel(oldapplex, oldappley, 0);
  matrix.drawPixel(myx, myy, APPLE_COLOR);
  oldapplex = myx;
  oldappley = myy;
  matrix.show();
}

void setapple() {
  bool valid = true;
  applex = random(0, 8);
  appley = random(0, 8);
  for (short c = 0; c < snake_count; c++) {
    for (short i = 0; i < snakes[0].len; i++) {
      valid = valid && ((applex != snakes[0].pos[i].x) || (appley != snakes[0].pos[i].y));
      if (!valid) {
        break;
      }
    }
  }
  while (!valid) {
    applex = random(0, 8);
    appley = random(0, 8);
    for (short c = 0; c < snake_count; c++) {
      for (short i = 0; i < snakes[0].len; i++) {
        valid = ((applex != snakes[0].pos[i].x) || (appley != snakes[0].pos[i].y));
        if (!valid) {
          break;
        }
      }
    }
  }
  setapplexy(applex, appley);
  if (mydelay > MINDELAY * SPEEDUP) {
    mydelay = (int)mydelay * SPEEDUP;
  }
}

void setup_snakes(int count) {
  lightsOff();
  snakes[0].pos[0].x = 2;
  snakes[0].pos[0].y = 2;
  snakes[0].len = 1;
  snakes[0].colors[0] = 0x07E0; snakes[0].colors[1] = 0x0600; snakes[0].colors[2] = 0x0500; snakes[0].colors[3] = 0x0400; snakes[0].colors[4] = 0x0300; snakes[0].colors[5] = 0x0200; snakes[0].colors[6] = 0x0100;
  snakes[0].dir = UP;
  dir[0] = UP;
  snake_is_dead[0] = false;
  if (count == 2) {
    snakes[1].pos[0].x = 6;
    snakes[1].pos[0].y = 6;
    snakes[1].len = 1;
    snakes[1].colors[0] = 0xF81D; snakes[1].colors[1] = 0xC016; snakes[1].colors[2] = 0x9811; snakes[1].colors[3] = 0x700C; snakes[1].colors[4] = 0x5009; snakes[1].colors[5] = 0x3005; snakes[1].colors[6] = 0x2004;
    snakes[1].dir = DOWN;
    dir[1] = DOWN;
    snake_is_dead[1] = false;
  }
  applex = oldapplex = 4;
  appley = oldappley = 4;
  setapplexy(applex, appley);
  mydelay = INITIALDELAY;
  last_moved = millis();
  buzzer_play_melody(snake_start, snake_start_durations, 9, NOFLUSH);
  snake_count = count;
}

void tictactoe_draw_selected() {
  if (millis() - last_toggled >= 333) {
    showing = !showing;
    if (showing) {
      matrix.fillRect(grid_starts[selected].x, grid_starts[selected].y, 2, 2, activecolor);
    } else {
      matrix.fillRect(grid_starts[selected].x, grid_starts[selected].y, 2, 2, grid[selected]);
    }
    last_toggled = millis();
    matrix.show();
  }
}

void tictactoe_move_selected(short joydirection) {
  if (joydirection == RIGHT && selected % 3 < 2) {
    if (showing) {
      matrix.fillRect(grid_starts[selected ].x, grid_starts[selected].y, 2, 2, grid[selected]);
      last_toggled = 0;
    }
    selected++;
    last_moved = millis();

  }
  else if (joydirection == LEFT && selected % 3 > 0) {
    if (showing) {
      matrix.fillRect(grid_starts[selected].x, grid_starts[selected].y, 2, 2, grid[selected]);
      last_toggled = 0;
    }
    selected--;
    last_moved = millis();

  }
  else if (joydirection == DOWN && selected < 6) {
    if (showing) {
      matrix.fillRect(grid_starts[selected].x, grid_starts[selected].y, 2, 2, grid[selected]);
      last_toggled = 0;
    }
    selected += 3;
    last_moved = millis();
  }
  else if (joydirection == UP && selected > 2) {
    if (showing) {
      matrix.fillRect(grid_starts[selected].x, grid_starts[selected].y, 2, 2, grid[selected]);
      last_toggled = 0;
    }
    selected -= 3;
    last_moved = millis();
  }

}

void tictactoe_checkproduct(short product) {
  if (product == 1) {
    winningplayer = 0;
    win = true;
  } else if (product == 8) {
    winningplayer = 1;
    win = true;
  }
}

void tictactoe_checkwin() {
  // inefficient, but readable
  // check rows
  tictactoe_checkproduct(wingrid[0]*wingrid[1]*wingrid[2]);
  tictactoe_checkproduct(wingrid[3]*wingrid[4]*wingrid[5]);
  tictactoe_checkproduct(wingrid[6]*wingrid[7]*wingrid[8]);
  // check columns
  tictactoe_checkproduct(wingrid[0]*wingrid[3]*wingrid[6]);
  tictactoe_checkproduct(wingrid[1]*wingrid[4]*wingrid[7]);
  tictactoe_checkproduct(wingrid[2]*wingrid[5]*wingrid[8]);
  // diagonals
  tictactoe_checkproduct(wingrid[0]*wingrid[4]*wingrid[8]);
  tictactoe_checkproduct(wingrid[6]*wingrid[4]*wingrid[2]);
}

void tictactoe_place_player(short player) {
  if (grid[selected] == BLACK) {
    grid[selected] = activecolor;
    wingrid[selected] = player + 1; // so a product of 0 would mean no winner, 1 = p1 wins, 8 = p2 wins
    matrix.fillRect(grid_starts[selected].x, grid_starts[selected].y, 2, 2, activecolor);
    matrix.show();
    count++;
    if (activeplayer == 0)
      buzzer_play_melody(tictactoe_p1, tictactoe_pdur, 1, FLUSH);
    else
      buzzer_play_melody(tictactoe_p2, tictactoe_pdur, 1, FLUSH);

    tictactoe_checkwin();
    if (win) {
      for (short i = 0; i < 9; i++) {
        if (wingrid[i] + winningplayer == 2)
          matrix.fillRect(grid_starts[i].x, grid_starts[i].y, 2, 2, WHITE);
      }
      matrix.show();
      buzzer_play_melody(snake_dead, snake_dead_durations, 7, FLUSH);
      delay(3000);
      choose_mode();
      return;

    } else if (count == 9) {
      for (short i = 0; i < 9; i++) {
        matrix.fillRect(grid_starts[i].x, grid_starts[i].y, 2, 2, WHITE);
      }
      buzzer_play_melody(snake_dead, snake_dead_durations, 7, FLUSH);
      matrix.show();
      delay(3000);
      choose_mode();
      return;
    }
    if (player == 0) {
      activecolor = MAGENTA;
      activeplayer = 1;
    }
    else {
      activecolor = GREEN;
      activeplayer = 0;
    }
    for(short i=0;i<9;i++){
      if(grid[i]==BLACK){
      selected=i;
      break;
      }
    }
  }

}
void setup_tictactoe() {
  lightsOff();
  matrix.drawLine(2, 0, 2, 7, TICTACTOEGRID) ;
  matrix.drawLine(5, 0, 5, 7, TICTACTOEGRID) ;
  matrix.drawLine(0, 2, 7, 2, TICTACTOEGRID) ;
  matrix.drawLine(0, 5, 7, 5, TICTACTOEGRID) ;
  selected = 0;
  count = 0;
  for (short i = 0; i < 9; i++) {
    grid[i] = BLACK;
    wingrid[i] = 10;
  }
  showing = false;
  last_toggled = 0;
  last_moved = millis();
  activecolor = GREEN;
  activeplayer = 0;
  win = false;
  tictactoe_draw_selected();
}

void show_snake2_mode() {
  matrix.fillRect(1, 1, 6, 6, BLACK);
  matrix.drawLine(2, 6, 2, 4, GREEN);
  matrix.drawLine(6, 2, 4, 2, MAGENTA);
  matrix.drawPixel(2, 2, RED);
  matrix.show();
}
void show_tictactoe_mode() {
  matrix.fillRect(1, 1, 6, 6, BLACK);
  matrix.drawRect(1, 1, 2, 2, GREEN);
  matrix.drawRect(3, 3, 2, 2, MAGENTA);
  matrix.drawRect(5, 5, 2, 2, GREEN);
  matrix.show();
}
void show_snake1_mode() {
  matrix.fillRect(1, 1, 6, 6, BLACK);
  matrix.drawLine(2, 6, 2, 4, GREEN);
  matrix.drawPixel(2, 2, RED);
  matrix.show();
}
void show_togglesound_mode() {
  matrix.fillRect(1, 1, 6, 6, BLACK);
  matrix.fillRect(2, 3, 2, 2, soundon ? GREEN : GREY);
  matrix.drawLine(4, 2, 4, 5, soundon ? GREEN : GREY);
  matrix.drawLine(5, 1, 5, 6, soundon ? GREEN : GREY);
  if (!soundon) {
    matrix.drawLine(1, 1, 6, 6, RED);
    matrix.drawLine(6, 1, 1, 6, RED);
  }
  matrix.show();
}

void buzzer_play_melody(int melody[], int durations[], int note_count, bool toflush) {
  if (!soundon) return;
  buzzer.playMelody(melody, durations, note_count);
  if (toflush)
    while (buzzer.getState() != BUZZER_IDLE) buzzer.loop();
}

void lightsOff() {
  matrix.fillScreen(0);
  matrix.show();
}

void choose_mode() {
  bool game_selected = false;
  lightsOff();
  matrix.drawRect(0, 0, 8, 8, BLUE);
  switch (mode) {
    case SNAKE2PLAYERS: show_snake2_mode(); break;
    case TICTACTOE: show_tictactoe_mode(); break;
    case SNAKE1PLAYER: show_snake1_mode(); break;
    case TOGGLESOUND: show_togglesound_mode(); break;
  }
  read_joysticks();
  while (!game_selected) {
    while (joy1_pushed == false) {
      if (joysticks_moved   && (millis() - last_moved >= 350)) {
        last_moved = millis();
        if (joy1_direction == RIGHT && mode < MAXMODE) {
          mode++;
          buzzer_play_melody(notify1, notify1_durations, 2, FLUSH);
        }
        if (joy1_direction == LEFT && mode > MINMODE) {
          mode--;
          buzzer_play_melody(notify1, notify1_durations, 2, FLUSH);
        }
        switch (mode) {
          case SNAKE2PLAYERS: show_snake2_mode(); break;
          case TICTACTOE: show_tictactoe_mode(); break;
          case SNAKE1PLAYER: show_snake1_mode(); break;
          case TOGGLESOUND: show_togglesound_mode(); break;
        }
      }
      read_joysticks();
    }
    if (mode == TOGGLESOUND) {
      soundon = !soundon;
      buzzer_play_melody(notify1, notify1_durations, 2, FLUSH);
      show_togglesound_mode();
      read_joysticks();
    } else {
      game_selected = true;
    }
  }
  buzzer_play_melody(notify1, notify1_durations, 2, FLUSH);
  delay(1000);
  switch (mode) {
    case SNAKE2PLAYERS:
      setup_snakes(2);
      break;
    case TICTACTOE:
      setup_tictactoe();
      break;
    case SNAKE1PLAYER:
      setup_snakes(1);
      break;
  }
}

void setup() {
  // SERIAL INITIALIZE
  Serial.begin(115200);
  while (!Serial) {}
  Serial.println("Started");
  pinMode(LED_PIN, OUTPUT);
  pinMode(PUSH1, INPUT_PULLUP);
  pinMode(PUSH2, INPUT_PULLUP);
  if (digitalRead(PUSH1) == LOW)
    oldStatePB1 = PRESSED;
  else oldStatePB1 = RELEASED;
  if (digitalRead(PUSH2) == LOW)
    oldStatePB2 = PRESSED;
  else oldStatePB2 = RELEASED;
  matrix.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  lightsOff();
  // gameboy startup animation
  matrix.drawLine(2, 0, 5, 0, BLUE);
  matrix.show();
  delay(200);
  matrix.drawLine(2, 0, 5, 0, BLACK);
  matrix.drawLine(2, 1, 5, 1, BLUE);
  matrix.show();
  delay(200);
  matrix.drawLine(2, 1, 5, 1, BLACK);
  matrix.drawLine(2, 2, 5, 2, BLUE);
  matrix.show();
  delay(200);
  matrix.drawLine(2, 2, 5, 2, BLACK);
  matrix.drawLine(2, 3, 5, 3, BLUE);
  matrix.show();
  delay(200);
  matrix.drawLine(2, 3, 5, 3, BLACK);
  matrix.drawLine(2, 4, 5, 4, BLUE);
  matrix.show();
  buzzer_play_melody(start, start_durations, 2, FLUSH);
  delay(1000);
  mode = SNAKE2PLAYERS;
  choose_mode();
  randomSeed(millis());
}

void color_snake() {
  short coloridx;
  for (short s = 0; s < snake_count; s++) {
    for (short i = snakes[s].len - 1, coloridx = 0; i >= 0; i--, coloridx++) {
      matrix.drawPixel(snakes[s].pos[i].x, snakes[s].pos[i].y, snakes[s].colors[coloridx]);
    }
  }
}
void color_dead_snake() {
  short coloridx;
  for (short s = 0; s < snake_count; s++) {
    if (snake_is_dead[s]) {
      for (short i = snakes[s].len - 1, coloridx = 0; i >= 0; i--, coloridx++) {
        matrix.drawPixel(snakes[s].pos[i].x, snakes[s].pos[i].y, WHITE);
      }
    }
  }
  matrix.show();
}

bool snake_collision() {
  bool collision = false;
  // check if each snake ate itself
  for (short s = 0; s < snake_count; s++) {
    if (snakes[s].len >= 4) {
      for (short i = 0; i < snakes[s].len - 2; i++) {
        if (snakes[s].pos[i].x == snakes[s].pos[snakes[s].len - 1].x &&  snakes[s].pos[i].y ==  snakes[s].pos[snakes[s].len - 1].y) {
          snake_is_dead[s] = true;
          collision = true;
          Serial.print("Snake ");
          Serial.print(s);
          Serial.println(" ate itself");
        }
      }
    }
  }
  if (snake_count == 2) {
    // check if 1 ate 2
    for (short i = 0; i < snakes[1].len ; i++) {
      if ((snakes[1].pos[i].x == snakes[0].pos[snakes[0].len - 1].x) &&  (snakes[1].pos[i].y ==  snakes[0].pos[snakes[0].len - 1].y)) {
        snake_is_dead[0] = true;
        collision = true;
        Serial.print("Snake 1 ate 2 i=");
        Serial.println(i);
      }
    }
    // check if 2 ate 1
    for (short i = 0; i < snakes[0].len; i++) {
      if ((snakes[0].pos[i].x == snakes[1].pos[snakes[1].len - 1].x) &&  (snakes[0].pos[i].y ==  snakes[1].pos[snakes[1].len - 1].y)) {
        snake_is_dead[1] = true;
        collision = true;
        Serial.print("Snake 2 ate 1 i=");
        Serial.println(i);
      }
    }
  }
  return collision;
}

void move_snake() {
  unsigned short head_old_x, head_old_y;
  bool apple_eaten = false;
  // for each snake
  for (short s = 0; s < snake_count; s++) {
    // we clear the tail
    matrix.drawPixel(snakes[s].pos[0].x, snakes[s].pos[0].y, 0);
    // we move the position of each segment in the arrays
    for (short i = 0; i < snakes[s].len - 1; i++) {
      snakes[s].pos[i].x = snakes[s].pos[i + 1].x;
      snakes[s].pos[i].y = snakes[s].pos[i + 1].y;
    }
    // we store the old head position
    head_old_x = snakes[s].pos[snakes[s].len - 1].x;
    head_old_y = snakes[s].pos[snakes[s].len - 1].y;
    // we move the head towards the (new/right) direction
    switch (dir[s]) {
      case UP: snakes[s].pos[snakes[s].len - 1].y = (head_old_y - 1) % 8; break;
      case DOWN: snakes[s].pos[snakes[s].len - 1].y = (head_old_y + 1) % 8; break;
      case LEFT: snakes[s].pos[snakes[s].len - 1].x = (head_old_x - 1) % 8; break;
      case RIGHT: snakes[s].pos[snakes[s].len - 1].x = (head_old_x + 1) % 8; break;
    }
  }
  // we check if the head collides with either the same snake, or the other one
  if (snake_collision()) {
    color_snake();
    color_dead_snake();
    buzzer_play_melody(snake_dead, snake_dead_durations, 7, FLUSH);
    delay(3000);
    choose_mode();
    return;
  }
  for (short s = 0; s < snake_count; s++) {
    // if we are here, there are no collisions. we check if the snake is eating the apple
    if (snakes[s].pos[snakes[s].len - 1].x == applex && snakes[s].pos[snakes[s].len - 1].y == appley ) {
      // if it's shorter than the display, we extend the snake
      if (snakes[s].len < 7) {
        snakes[s].len += 1;
        // ..in the right direction
        switch (dir[s]) {
          case UP: snakes[s].pos[snakes[s].len - 1].y = (snakes[s].pos[snakes[s].len - 2].y - 1) % 8; snakes[s].pos[snakes[s].len - 1].x = snakes[s].pos[snakes[s].len - 2].x; break;
          case DOWN: snakes[s].pos[snakes[s].len - 1].y = (snakes[s].pos[snakes[s].len - 2].y + 1) % 8; snakes[s].pos[snakes[s].len - 1].x = snakes[s].pos[snakes[s].len - 2].x; break;
          case LEFT: snakes[s].pos[snakes[s].len - 1].x = (snakes[s].pos[snakes[s].len - 2].x - 1) % 8; snakes[s].pos[snakes[s].len - 1].y = snakes[s].pos[snakes[s].len - 2].y; break;
          case RIGHT: snakes[s].pos[snakes[s].len - 1].x = (snakes[s].pos[snakes[s].len - 2].x + 1) % 8; snakes[s].pos[snakes[s].len - 1].y = snakes[s].pos[snakes[s].len - 2].y ; break;
        }
      }
      apple_eaten = true;
    }
  }
  /*
    // we check if the head collides with either the same snake, or the other one
    if (snake_collision()) {
    color_snake();
    color_dead_snake();
    buzzer_play_melody(snake_dead, snake_dead_durations, 7, FLUSH);
    delay(3000);
    choose_mode();
    return;
    }
  */
  // new apple in random spot if apple was eaten
  if (apple_eaten) {
    buzzer_play_melody(snake_eaten_apple, snake_eaten_apple_durations, 4, NOFLUSH);
    setapple();
  }
  // now we draw the snakes. if we are here, there were no collisions
  color_snake();
  matrix.show();
}

void read_joysticks() {
  if ((joystick_1.read(X)) > 600) {
    joysticks_moved = true;
    joy1_moved = true;
    joy1_direction = RIGHT;
    Serial.println("Joy1 RIGHT");
  }
  else if ((joystick_1.read(X)) < 400) {
    joysticks_moved = true;
    joy1_moved = true;
    joy1_direction = LEFT;
    Serial.println("Joy1 LEFT");
  }
  else if ((joystick_1.read(Y)) > 600) {
    joysticks_moved = true;
    joy1_moved = true;
    joy1_direction = DOWN;
    Serial.println("Joy1 DOWN");
  }
  else if ((joystick_1.read(Y)) < 400) {
    joysticks_moved = true;
    joy1_moved = true;
    joy1_direction = UP;
    Serial.println("Joy1 UP");
  }
  else {
    joy1_moved = false;
    joy1_direction = CENTER;
  }
  if ((joystick_2.read(X)) > 600) {
    joysticks_moved = true;
    joy2_moved = true;
    joy2_direction = RIGHT;
    Serial.println("Joy2 RIGHT");
  }
  else if ((joystick_2.read(X)) < 400) {
    joysticks_moved = true;
    joy2_moved = true;
    joy2_direction = LEFT;
    Serial.println("Joy2 LEFT");
  }
  else if ((joystick_2.read(Y)) > 600) {
    joysticks_moved = true;
    joy2_moved = true;
    joy2_direction = DOWN;
    Serial.println("Joy2 DOWN");
  }
  else if ((joystick_2.read(Y)) < 400) {
    joysticks_moved = true;
    joy2_moved = true;
    joy2_direction = UP;
    Serial.println("Joy2 UP");
  }
  else {
    joy2_moved = false;
    joy2_direction = CENTER;
  }
  if (!joy1_moved && !joy2_moved) joysticks_moved = false;

  if (digitalRead(PUSH1) == LOW) {
    if (oldStatePB1 == RELEASED) {
      oldStatePB1 = PRESSED;
    }
  } else {
    if (oldStatePB1 == PRESSED) {
      joy1_pushed = true;
      Serial.println("Joy1 pushed and released");
      oldStatePB1 = RELEASED;
    } else {
      joy1_pushed = false;
    }
  }
  if (digitalRead(PUSH2) == LOW) {
    if (oldStatePB2 == RELEASED) {
      oldStatePB2 = PRESSED;
    }
  } else {
    if (oldStatePB2 == PRESSED) {
      joy2_pushed = true;
      Serial.println("Joy2 pushed and released");
      oldStatePB2 = RELEASED;
    } else {
      joy2_pushed = false;
    }
  }

}


void check_apply_moves() {
  switch (mode) {
    case SNAKE2PLAYERS:
      if (joy1_direction == RIGHT) {
        if (dir[0] != LEFT) dir[0] = RIGHT;
      } else if (joy1_direction == LEFT) {
        if (dir[0] != RIGHT) dir[0] = LEFT;
      }
      else if (joy1_direction == DOWN) {
        if (dir[0] != UP) dir[0] = DOWN;
      }
      else if (joy1_direction == UP) {
        if (dir[0] != DOWN) dir[0] = UP;
      }
      if (joy2_direction == RIGHT) {
        if (dir[1] != LEFT) dir[1] = RIGHT;
      } else if (joy2_direction == LEFT) {
        if (dir[1] != RIGHT) dir[1] = LEFT;
      }
      else if (joy2_direction == DOWN) {
        if (dir[1] != UP) dir[1] = DOWN;
      }
      else if (joy2_direction == UP) {
        if (dir[1] != DOWN) dir[1] = UP;
      }
      if (millis() - last_moved >= mydelay) {
        move_snake();
        last_moved = millis();
      }
      break;
    case TICTACTOE:
      tictactoe_draw_selected();
      if (millis() - last_moved >= 350) {
        if (activeplayer == 0) {
          if (joy1_direction != CENTER) {
            tictactoe_move_selected(joy1_direction);
          }
          if (joy1_pushed) {
            tictactoe_place_player(0);
          }
        } else {
          if (joy2_direction != CENTER) {
            tictactoe_move_selected(joy2_direction);
          }
          if (joy2_pushed) {
            tictactoe_place_player(1);
          }

        }
      }
      break;
    case SNAKE1PLAYER:
      if (joy1_direction == RIGHT) {
        if (dir[0] != LEFT) dir[0] = RIGHT;
      } else if (joy1_direction == LEFT) {
        if (dir[0] != RIGHT) dir[0] = LEFT;
      }
      else if (joy1_direction == DOWN) {
        if (dir[0] != UP) dir[0] = DOWN;
      }
      else if (joy1_direction == UP) {
        if (dir[0] != DOWN) dir[0] = UP;
      }
      if (joy2_direction == RIGHT) {
        if (dir[0] != LEFT) dir[0] = RIGHT;
      } else if (joy2_direction == LEFT) {
        if (dir[0] != RIGHT) dir[0] = LEFT;
      }
      else if (joy2_direction == DOWN) {
        if (dir[0] != UP) dir[0] = DOWN;
      }
      else if (joy2_direction == UP) {
        if (dir[0] != DOWN) dir[0] = UP;
      }
      if (millis() - last_moved >= mydelay) {
        move_snake();
        last_moved = millis();
      }
      break;
  }
}

void loop() {
  buzzer.loop();
  read_joysticks();
  check_apply_moves();

}
